@@ -1,85 +1,23 @@
 /*
  *  File ergm/src/changestats.h
- *  Part of the statnet package, http://statnetproject.org
+ *  Part of the statnet package, http://statnet.org
  *
  *  This software is distributed under the GPL-3 license.  It is free,
  *  open source, and has the attribution requirements (GPL Section 7) in
- *    http://statnetproject.org/attribution
+ *    http://statnet.org/attribution
  *
- *  Copyright 2010 the statnet development team
+ *  Copyright 2012 the statnet development team
  */
-
 #ifndef CHANGESTATS_H
 #define CHANGESTATS_H
 
 #include "edgetree.h"
+#include "changestat.h"
 
-typedef struct ModelTermstruct {
-	void (*d_func)(Edge, Vertex*, Vertex*, struct ModelTermstruct*, Network*);
-  	void (*s_func)(struct ModelTermstruct*, Network*);
-        void (*t_func)(struct ModelTermstruct*, Network*);
-	double *attrib; /* Ptr to vector of covariates (if necessary; generally unused) */
-	int nstats;   /* Number of change statistics to be returned */
-	double *dstats; /* ptr to change statistics returned */
-	int ninputparams; /* Number of input parameters passed to function */
-	double *inputparams; /* ptr to input parameters passed */
-} ModelTerm;
-
-
-/****************************************************
- Macros to make life easier                         *
- Note:  These things still need to be documented    */ 
-#define CHOOSE(n,r) ((n)<(r) ? (0) : (my_choose((double)(n),(int)(r)))) 
-
-#define IS_OUTEDGE(a,b) (EdgetreeSearch((a),(b),nwp->outedges)!=0?1:0)
-#define IS_INEDGE(a,b) (EdgetreeSearch((a),(b),nwp->inedges)!=0?1:0)
-#define IS_UNDIRECTED_EDGE(a,b) IS_OUTEDGE(MIN(a,b), MAX(a,b))
-#define MIN_OUTEDGE(a) (EdgetreeMinimum(nwp->outedges, (a)))
-#define MIN_INEDGE(a) (EdgetreeMinimum(nwp->inedges, (a)))
-#define NEXT_OUTEDGE(e) (EdgetreeSuccessor(nwp->outedges,(e)))
-#define NEXT_INEDGE(e) (EdgetreeSuccessor(nwp->inedges,(e)))
-#define OUTVAL(e) (nwp->outedges[(e)].value)
-#define INVAL(e) (nwp->inedges[(e)].value)
-#define TOGGLE(a,b) (ToggleEdge((a),(b),nwp));
-#define TOGGLE_DISCORD(a,b) (ToggleEdge((a),(b),nwp+1));
-
-#define STEP_THROUGH_OUTEDGES(a,e,v) for((e)=MIN_OUTEDGE(a);((v)=OUTVAL(e))!=0;(e)=NEXT_OUTEDGE(e))
-#define STEP_THROUGH_INEDGES(a,e,v) for((e)=MIN_INEDGE(a);((v)=INVAL(e))!=0;(e)=NEXT_INEDGE(e))
-
-#define N_NODES (nwp->nnodes)
-#define N_DYADS (nwp->directed_flag?(nnodes*(nnodes-1)):nwp->bipartite?nwp->bipartite*(nnodes-nwp->bipartite):(nnodes*(nnodes-1)/2))
-#define OUT_DEG (nwp->outdegree)
-#define IN_DEG (nwp->indegree)
-#define DIRECTED (nwp->directed_flag)
-#define BIPARTITE (nwp->bipartite)
-#define N_EDGES (nwp->nedges)
-#define NEXT_INEDGE_NUM (nwp->next_inedge)
-#define NEXT_OUTEDGE_NUM (nwp->next_outedge)
-
-#define N_CHANGE_STATS (mtp->nstats)
-#define INPUT_PARAM (mtp->inputparams)
-#define CHANGE_STAT (mtp->dstats)
-#define INPUT_ATTRIB (mtp->attrib)
-#define N_INPUT_PARAMS (mtp->ninputparams)
-
-#define ZERO_ALL_CHANGESTATS(a) for((a)=0; (a)<N_CHANGE_STATS; (a)++) CHANGE_STAT[(a)]=0.0
-#define FOR_EACH_TOGGLE(a) for((a)=0; (a)<ntoggles; (a)++)
-#define TOGGLE_IF_MORE_TO_COME(a) if((a)+1<ntoggles) TOGGLE(heads[(a)],tails[(a)])
-#define TOGGLE_DISCORD_IF_MORE_TO_COME(a) if((a)+1<ntoggles) TOGGLE_DISCORD(heads[(a)],tails[(a)])
-#define UNDO_PREVIOUS_TOGGLES(a) (a)--; while(--(a)>=0) TOGGLE(heads[(a)],tails[(a)])
-#define UNDO_PREVIOUS_DISCORD_TOGGLES(a) (a)--; while(--(a)>=0) {TOGGLE(heads[(a)],tails[(a)]); TOGGLE_DISCORD(heads[(a)],tails[(a)])}
-
-/****************************************************/
-/* changestat function prototypes, 
-   plus a few supporting function prototypes */
-#define CHANGESTAT_FN(a) void (a) (Edge ntoggles, Vertex *heads, Vertex *tails, ModelTerm *mtp, Network *nwp)
-/* NB:  CHANGESTAT_FN is now deprecated (replaced by D_CHANGESTAT_FN) */
-#define D_CHANGESTAT_FN(a) void (a) (Edge ntoggles, Vertex *heads, Vertex *tails, ModelTerm *mtp, Network *nwp)
-#define T_CHANGESTAT_FN(a) void (a) (ModelTerm *mtp, Network *nwp)
-#define S_CHANGESTAT_FN(a) void (a) (ModelTerm *mtp, Network *nwp)
 /********************  changestats:  A    ***********/
 D_CHANGESTAT_FN(d_absdiff);
 D_CHANGESTAT_FN(d_absdiffcat);
+D_CHANGESTAT_FN(d_adegcor); S_CHANGESTAT_FN(s_adegcor);
 D_CHANGESTAT_FN(d_altkstar);
 D_CHANGESTAT_FN(d_asymmetric);
 /********************  changestats:  B    ***********/
@@ -104,11 +42,13 @@ D_CHANGESTAT_FN(d_boundeddegree);
 D_CHANGESTAT_FN(d_boundedidegree);
 D_CHANGESTAT_FN(d_boundedodegree);
 D_CHANGESTAT_FN(d_boundedistar);
-  double my_choose(double n, int r);
 D_CHANGESTAT_FN(d_boundedkstar);
 D_CHANGESTAT_FN(d_boundedostar);
 D_CHANGESTAT_FN(d_boundedtriangle);
-  Vertex CountTriangles (Vertex h, Vertex t, int outcount, 
+
+/* *** don't forget tail-> head, so this function accepts tail first, not head */
+
+  Vertex CountTriangles (Vertex tail, Vertex head, int outcount, 
                          int incount, Network *nwp);
 /********************  changestats:  C    ***********/
 D_CHANGESTAT_FN(d_concurrent);
@@ -118,9 +58,15 @@ D_CHANGESTAT_FN(d_cycle);
   void edgewise_path_recurse(Network *g, Vertex dest, 
      Vertex curnode, Vertex *availnodes, long int availcount, 
      long int curlen, double *countv, long int maxlen);
-  void edgewise_cycle_census(Network *g, Vertex t, Vertex h, 
+
+/* *** I didn't swap heads and tails here, since these already 
+   seem in line with the tails->heads naming convenction*/
+       
+  void edgewise_cycle_census(Network *g, Vertex tail, Vertex head, 
      double *countv, long int maxlen);
 /********************  changestats:  D    ***********/
+D_CHANGESTAT_FN(d_degcor); S_CHANGESTAT_FN(s_degcor);
+D_CHANGESTAT_FN(d_degcrossprod);
 D_CHANGESTAT_FN(d_degree);
 D_CHANGESTAT_FN(d_degree_by_attr);
 D_CHANGESTAT_FN(d_degree_w_homophily);
@@ -151,7 +97,6 @@ D_CHANGESTAT_FN(d_gwtesp);
 D_CHANGESTAT_FN(d_gwtnsp);
 /********************  changestats:   H    ***********/
 D_CHANGESTAT_FN(d_hamming);
-D_CHANGESTAT_FN(d_hamming_weighted);
 D_CHANGESTAT_FN(d_hammingmix_constant);
 D_CHANGESTAT_FN(d_hammingmix);
 /********************  changestats:   I    ***********/
@@ -160,6 +105,7 @@ D_CHANGESTAT_FN(d_idegree_by_attr);
 D_CHANGESTAT_FN(d_idegree_w_homophily);
 D_CHANGESTAT_FN(d_intransitive);
 D_CHANGESTAT_FN(d_isolates);
+S_CHANGESTAT_FN(s_isolates);
 D_CHANGESTAT_FN(d_istar);
 /********************  changestats:   K    ***********/
 D_CHANGESTAT_FN(d_kstar);
@@ -170,6 +116,7 @@ D_CHANGESTAT_FN(d_m2star);
 D_CHANGESTAT_FN(d_meandeg);
 D_CHANGESTAT_FN(d_mix);
 D_CHANGESTAT_FN(d_mutual);
+D_CHANGESTAT_FN(d_mutual_by_attr);
 /********************  changestats:   N    ***********/                       
 D_CHANGESTAT_FN(d_nearsimmelian);
 D_CHANGESTAT_FN(d_nodecov);
@@ -185,8 +132,13 @@ D_CHANGESTAT_FN(d_nsp);
 D_CHANGESTAT_FN(d_odegree);
 D_CHANGESTAT_FN(d_odegree_by_attr);
 D_CHANGESTAT_FN(d_odegree_w_homophily);
+D_CHANGESTAT_FN(d_opentriad);
 D_CHANGESTAT_FN(d_ostar);
+D_CHANGESTAT_FN(d_outdegreepopularity);
+/********************  changestats:   P    ***********/
+D_CHANGESTAT_FN(d_pdegcor); S_CHANGESTAT_FN(s_pdegcor);
 /********************  changestats:   R    ***********/
+D_CHANGESTAT_FN(d_rdegcor); S_CHANGESTAT_FN(s_rdegcor);
 D_CHANGESTAT_FN(d_receiver);
 /********************  changestats:   S    ***********/
 D_CHANGESTAT_FN(d_sender);
@@ -200,10 +152,12 @@ D_CHANGESTAT_FN(d_tesp);
 D_CHANGESTAT_FN(d_threepath);
 D_CHANGESTAT_FN(d_tnsp);
 D_CHANGESTAT_FN(d_transitive);
+D_CHANGESTAT_FN(d_transitiveties); S_CHANGESTAT_FN(s_transitiveties);
 D_CHANGESTAT_FN(d_triadcensus);
 D_CHANGESTAT_FN(d_triangle);
 D_CHANGESTAT_FN(d_tripercent);
 D_CHANGESTAT_FN(d_ttriple);
+D_CHANGESTAT_FN(d_transitiveties);
 D_CHANGESTAT_FN(d_edges_i); /* Michael */
 D_CHANGESTAT_FN(d_arcs_i); /* Michael */
 D_CHANGESTAT_FN(d_arcs_j); /* Michael */
@@ -216,5 +170,7 @@ D_CHANGESTAT_FN(d_ctriple_ijk); /* Michael */
 D_CHANGESTAT_FN(d_twostar_i); /* Michael */
 D_CHANGESTAT_FN(d_twostar_ijk); /* Michael */
 
+
+
               
 #endif
